@@ -75,9 +75,18 @@ class BKTreeNode[T]:
 
 class BKTree[T]:
 
-    def __init__(self, distance_function: Callable[[Sequence[Hashable], Sequence[Hashable]], int], max_distance: int = 0,
-                 key: Callable[[T], str] | None = None, value_type: type[T] = str, node_type: type[BKTreeNode] = BKTreeNode,
-                 *, post_add_hook: Callable[[Self, BKTreeNode], None] = default_post_add_hook):
+    def __init__(self,
+                 distance_function: Callable[[Sequence[Hashable], Sequence[Hashable]], int],
+                 max_distance: int = 0,
+                 key: Callable[[T], str] | None = None,
+                 value_type: type[T] = str,
+                 node_type: type[BKTreeNode] = BKTreeNode,
+                 *,
+                 post_add_hook: Callable[[Self, BKTreeNode], None] = lambda tree, node: None,
+                 post_already_added_hook: Callable[[Self, BKTreeNode], None] = lambda tree, node: None,
+                 post_filter_add_hook: Callable[[Self, BKTreeNode], None] = lambda tree, node: None,
+                 post_insert_subtree_hook: Callable[[Self, BKTreeNode], None] = lambda tree, node: None,
+                 post_combine_hook: Callable[[Self, BKTreeNode], None] = lambda tree, node: None):
         self.root: BKTreeNode[T] | None = None
         self.distance_function = distance_function
         self.max_distance = max_distance
@@ -86,6 +95,10 @@ class BKTree[T]:
         self.value_type = value_type
         self.node_type = node_type
         self.post_add_hook = post_add_hook
+        self.post_already_added_hook = post_already_added_hook
+        self.post_filter_add_hook = post_filter_add_hook
+        self.post_insert_subtree_hook = post_insert_subtree_hook
+        self.post_combine_hook = post_combine_hook
 
     @classmethod
     def from_json(cls, filepath: os.PathLike | str, distance_function: Callable[[Sequence[Hashable], Sequence[Hashable]], int],
@@ -248,6 +261,7 @@ class BKTree[T]:
                     return search_result[0], False
                 else:
                     logger.debug(f"{self.key(search_result[0].value)} is similar to {self.key(value)} and is already in the tree.")
+                    self.post_already_added_hook(self, search_result[0])
                     return search_result[0], False
 
     def _add(self, node: BKTreeNode, value: T) -> BKTreeNode:
@@ -267,6 +281,7 @@ class BKTree[T]:
         else:
             new_node = BKTreeNode(value, self.value_type)
             node.children[comparison_distance] = new_node
+            self.nb_elements += 1
             self.post_add_hook(self, new_node)
             logger.debug(
                 f"New song: {self.key(value)} as a children of {self.key(node.value)} (dist: {comparison_distance})"
@@ -284,8 +299,10 @@ class BKTree[T]:
                 if filtered_tree.root is None:
                     filtered_tree.root = BKTreeNode(node.value, self.value_type)
                     filtered_tree.nb_elements = 1
+                    added_node = filtered_tree.root
                 else:
-                    filtered_tree._add(filtered_tree.root, node.value)
+                    added_node = filtered_tree._add(filtered_tree.root, node.value)
+                self.post_filter_add_hook(filtered_tree, added_node)
 
             for _, child in node.children.items():
                 _filter_recursive(child)
@@ -421,7 +438,9 @@ class BKTree[T]:
             parent_node:
                 The parent node of the root of the subtree.
         """
-        self.insert(node.value, parent_node)
+        new_node = self.insert(node.value, parent_node)
+        if new_node:
+            self.post_insert_subtree_hook(self, new_node)
 
         for _, child in node.children.items():
             self.insert_subtree(child, parent_node)
@@ -523,7 +542,10 @@ class BKTree[T]:
             self._combine(other_tree.root)
 
     def _combine(self, other_tree_node: BKTreeNode):
-        self.add(other_tree_node.value)
+        new_node, _ = self.add(other_tree_node.value)
+
+        if new_node:
+            self.post_combine_hook(self, new_node)
 
         for node in other_tree_node.children.values():
             self._combine(node)
